*******************************************************
*  													  *
*  How to undertake an external validation study to   *
*  evaluate the performance of a clinical prediction  *
*  model - 2022										  *
*  													  *
*  Survival outcome example - Stata 16				  *
*  													  *
*  RDR, LA, KIES, JE, PD, GPM, LJB, GSC				  *
*  													  *
*******************************************************

*set working directory
cd "C:\Users\archerla\OneDrive - University of Birmingham\REVAMP\BMJ validation series\Part 2" 

*read in German breast cancer data, from webuse brcancer

use "Practical_brcancer_val", clear 

*set as survival data
stset rectime, failure(censrec==1) scale(365.25)

*categorise size to correspond to model equation groupings
gen size_cat=1 if size<=20
	replace size_cat=2 if size>20 & size<=50
	replace size_cat=3 if size>50 & size!=.

*generate variable for linear predictor of prediction model
gen lin_pred = - 0.0238523*((age/10)^3 -175.384857) ///
			  + 0.0108421*((age/10)^3*ln(age/10)-302.0701764) ///
			  + 0.2616501*meno ///
			  + 0.1922359*[size_cat==2] + 0.4210304*[size_cat==3] ///
			  + 0.2538847*[grade==3] ///
			  + 0.4084642*(ln(nodes/10) + 0.6478745614) ///
			  - 0.0003237*(er-165.0789133) ///
			  - 0.4136715*hormon ///
			  - 0.4859647 // all patients had chemotherapy
	
*transform to probability scale (baseline survival at 5 years = 0.507)
gen pred_surv_prob=0.507^exp(lin_pred)
gen pred_event_prob = 1 -pred_surv_prob 

*******************************************************

*~._.~*~._.~*~._.~*

*plot total LP distribution with summary statistics

sum lin_pred, detail
	local mean = r(mean)
	local median = r(p50)
	local sd = r(sd)
	local lq = r(p25)
	local uq = r(p75)

twoway (hist lin_pred, dens width(0.1) color(navy%60)) ///
		(kdensity  lin_pred , lcolor(navy) bwidth(0.35)), ///
		xtitle("Linear predictor value") scheme(s1color) ylabel(0(0.2)1) ///
		text(1 1 ///
		"Mean = `: display %3.2f `mean''" ///
		"Median = `: display %3.2f `median''" ///
		"SD = `: display %3.2f `sd''" ///
		"IQR = `: display %3.2f `lq'' to `: display %3.2f `uq''" ///
		, place(se) just(left))	legend(off)	

graph export "Reviewers updates\surv_LPDist_all.png", as(png) replace		

*~._.~*~._.~*~._.~*


*~._.~*~._.~*~._.~*

*plot LP distribution by outcome, with summary statistics

sum lin_pred if _d == 0, detail
	local mean_0 = r(mean)
	local median_0 = r(p50)
	local sd_0 = r(sd)
	local lq_0 = r(p25)
	local uq_0 = r(p75)
	
sum lin_pred if _d == 1, detail
	local mean_1 = r(mean)
	local median_1 = r(p50)
	local sd_1 = r(sd)
	local lq_1 = r(p25)
	local uq_1 = r(p75)

twoway  (hist lin_pred if _d == 0, dens color(navy%40) lcolor(black) width(0.1)) ///
		(hist lin_pred if _d == 1, dens color(dkgreen%40) lcolor(black) width(0.1)) ///
		(kdensity  lin_pred if _d == 0, lcolor(navy) bwidth(0.35)) ///
		(kdensity  lin_pred if _d == 1, lcolor(dkgreen) bwidth(0.35)), ///
		xtitle("Linear predictor value") scheme(s1color) ///
		legend(order(1 "No recurrence within 5 years" 2 "Recurrence within 5 years" )) ///
		text(1 0.8 ///
		"No recurrence:" ///
		"  Mean = `: display %3.2f `mean_0''" ///
		"  Median = `: display %3.2f `median_0''" ///
		"  SD = `: display %3.2f `sd_0''" ///
		"  IQR = `: display %3.2f `lq_0'' to `: display %3.2f `uq_0''" ///
		" " ///
		"Recurrence:" ///
		"  Mean = `: display %3.2f `mean_1''" ///
		"  Median = `: display %3.2f `median_1''" ///
		"  SD = `: display %3.2f `sd_1''" ///
		"  IQR = `: display %3.2f `lq_1'' to `: display %3.2f `uq_1''" ///		
		, place(se) just(left))		

graph export "Reviewers updates\surv_LPDist_byoutcome.png", as(png) replace		

*~._.~*~._.~*~._.~*


*******************************************************	
	
*calibration plot using pmcalplot (spike plot)
pmcalplot pred_event_prob, surv timepoint(5) lp(lin_pred) tte(20) ci nostat 	
	
*generate calibration plot with histogram

global grps 20 
egen groups = cut(lin_pred), group($grps)
replace groups = groups + 1
local k pseudo_gen

forvalues i = 1/$grps {
	* generate pseudo values at 5 years
    stpci if groups == `i', at(5) gen(pseudo`i'_`k'_d) 
}


* combine the pseudo observation into one column
gen pseudo = .
global grps 20 
local k pseudo_gen
forvalues i = 1/$grps {
    replace pseudo = pseudo`i'_`k'_d if groups == `i'
}

egen groups2 = cut(lin_pred), group(10)

bysort groups2: egen ave_exp = mean(pred_event_prob)
bysort groups2: egen ave_obs = mean(pseudo)

*generate with histogram

gen event = 0
replace event = 1 if _d == 1 & _t <= 1826.25

gen histeventn = . 
gen histnoneventn = .
gen histbinvar = 1

local bin = 20
local diff = 1/`bin'
local grp = 1
forvalues i = 0(`diff')1 {
	local difflow = `i'
	local diffhigh = `i'+`diff'
	replace histbinvar = `grp' if pred_event_prob>=`difflow' & pred_event_prob<`diffhigh'
	local grp = `grp'+1
}

forvalues i = 1/`bin' {
	qui count if event ==1 & histbinvar==`i' 
	qui replace histeventn=r(N) if histbinvar==`i' 
	qui count if event ==0 & histbinvar==`i' 
	qui replace histnoneventn=r(N) if histbinvar==`i'
	}
	
*scale to fit along the bottom of the calibration plot	
qui count if event == 1 
	local totale = r(N)
qui count if event == 0 
	local totalne = r(N)

qui replace histeventn = (histeventn/`totale')
qui replace histnoneventn = (histnoneventn/`totalne')

gen histexp = `diff'*histbinvar
gen droplinebase = -0.1

*scale to fit along the bottom of the calibration plot	
qui sum histeventn
	local maxe = r(max)
qui sum histnoneventn
	local maxne = r(max)
	local maxout = max(`maxe', `maxne')

qui replace histnoneventn = -1*histnoneventn

qui replace histeventn = droplinebase + (histeventn/`maxout')*(0.1)
qui replace histnoneventn = droplinebase +(histnoneventn/`maxout')*(0.1)

qui running pseudo pred_event_prob , span(1) ci generate(smooth) gense(smooth_se) replace nog
	gen smooth_lci = smooth - (1.96*smooth_se)
	replace smooth_lci=0 if smooth_lci<0
	gen smooth_uci = smooth + (1.96*smooth_se)
	replace smooth_uci=1 if smooth_uci>1
	
gen lb_ave_obs = .
gen ub_ave_obs = .

forvalues i = 0/9 {
	sts list if groups2==`i' , at(0 5) saving(temp, replace)
	preserve 
		use "temp", clear
		drop if time==0
		local cal_obs = 1 - survivor
		local cal_obs_lb=1-lb
		local cal_obs_ub=1-ub
	restore
		replace ave_obs = `cal_obs' if groups2==`i'
		replace lb_ave_obs = `cal_obs_lb' if groups2==`i'
		replace ub_ave_obs = `cal_obs_ub' if groups2==`i'
}
	
twoway (line smooth pred_event_prob if smooth <1, sort lcol(midblue)) ///
	(rarea smooth_lci smooth_uci pred_event_prob, sort fc(midblue%20) lc(midblue%20)) ///
	(scatter ave_obs ave_exp, msymbol(Oh) mcol(dkgreen)) ///
	(rspike ub_ave_obs lb_ave_obs ave_exp, lcol(dkgreen)) ///
	(rbar histeventn droplinebase histexp if (histexp< 1) & (histexp> 0), barw(0.05) msym(i) yline(-0.1, lw(vthin)) text(-.05 1 "Recurrence", place(w) size(vsmall)) lw(none) col(maroon)) ///
	(rbar histnoneventn droplinebase histexp if (histexp< 1) & (histexp> 0), barw(0.05) msym(i) text(-0.15 1 "No recurrence", place(w) size(vsmall)) lw(none) col(forest_green)) ///
	(function y = x, ra(0 1) clpat(-) clcol(gs5)) ///
	, ylabel(0(0.2)1, labsize(small)) xlabel(0(0.2)1, labsize(small)) ///
	aspect(1.2) yscale(range(-0.2 1)) xscale(range(0 1)) ///
	ytitle("Observed risk", size(small)) xtitle("Estimated risk", size(small)) title("") ///
	graphregion(color(white))  ///
	legend(on pos(3) col(1) order(3 "Groups"  4 "95% confidence interval - group" 1 "Calibration curve" 2 "95% confidence interval - curve" 7 "Ideal calibration") size(vsmall))
			
graph export "Calplots\survival_calplot.png", as(png) replace
	
**********************************************************************

*decision curve analysis
stdca pred_event_prob, timepoint(5) prob(yes) smooth scheme(s1color) ///
lcolor(black gs8 dkgreen) lpattern(dash solid solid) ///
legend(col(1) order(1 "Treat all" 2 "Treat none" 3 "Treat per model")) ///
	saving("survival_dca", replace)
	
use "survival_dca", clear

graph export "survival_dca.png", as(png) replace		
	

**********************************************************************	

*predictive performance statistics 

*Calibration slope
stcox lin_pred, nohr

*~._.~*~._.~*~._.~*

*R2d

str2d stcox lin_pred

*~._.~*~._.~*~._.~*

*Brier score

stbrier lin_pred , btime(5)


*~._.~*~._.~*~._.~*

*Harrell's C-statistic

*for use in somersd command
gen censind =1-_d if _st==1 
gen invhr =1/pred_event_prob 
					
somersd _t invhr if _st==1 , cenind(censind) tdist transf(c)

*~._.~*~._.~*~._.~*

*observed survival probabiity at 5 years
sts list, at(0 5)

*using event probabilities
local obs = 1-0.4916
local exp = 1-0.5995

sum rectime
	local N = r(N)
di "O/E = " `obs'/`exp'

local OE =  `obs'/`exp'
local se_ln_oe = sqrt((1-`obs')/(`N')*`obs')

di "95% ci = " exp(ln(`OE') - 1.96*`se_ln_oe') " to "exp(ln(`OE') + 1.96*`se_ln_oe')

*~._.~*~._.~*~._.~*

*Integrated Calibration Index (ICI)
running pseudo pred_event_prob , generate(p_cal)  
gen ici = abs(p_cal - pred_event_prob)
sum ici
di "95% ci" r(mean) - 1.96*(r(sd)/sqrt(r(N))) " to " r(mean) + 1.96*(r(sd)/sqrt(r(N)))

*~._.~*~._.~*~._.~*

*R for time-dependant AUROC using timeROC package
save "Survival_ValData.dta", replace
outsheet using "Survival_ValData.csv", comma replace

******************************************************************************